@@ -6,6 +6,11 @@ packageDescription <- function(pkg, lib.loc=NULL, fields=NULL, drop=TRUE)
         retval[fields] <- NA
     }
 
+    if(system.file(package = pkg, lib.loc = lib.loc) == "") {
+        warning("No package ", sQuote(pkg), " was found\n")
+        return(NA)
+    }
+
     file <- system.file("DESCRIPTION", package = pkg, lib.loc = lib.loc)
 
     if(file != "") {
@@ -19,8 +24,8 @@ packageDescription <- function(pkg, lib.loc=NULL, fields=NULL, drop=TRUE)
     }
 
     if((file == "") || (length(retval) == 0)){
-        warning(paste("DESCRIPTION file of package ", sQuote(pkg),
-                      " missing or broken\n"))
+        warning("DESCRIPTION file of package ", sQuote(pkg),
+                " is missing or broken\n")
         return(NA)
     }
 
